/*
// these are C type defines, the following should be true:
//
//   Bool     =>  true or false value
//   Byte     =>  8bit signed int
//   uByte    =>  8bit unsigned int
//   Char     =>  character signed value
//   uChar    =>  character unsigned value
//   Short    =>  16bit signed int
//   uShort   =>  16bit unsigned int
//   Int      =>  32bit signed int
//   uInt     =>  32bit unsigned int
//   Long     =>  64bit signed int (where possible)
//   uLong    =>  64bit unsigned int (where possible)
//   Float    =>  32bit signed float
//   Double   =>  64bit signed float
//
// Defining USE_LARGE_FLOATS and USE_LARGE_NUMBERS will change Long and
// Float to be larger, although this is not recommended unless you know
// your OS and system can handle larger 64 bit + words.
//
// As machines advance we may need to change these #if's to handle larger
// than 64 bits, for now they assume 64bits is the heavenly bit in the sky.
*/

#define DTYPES /* zlib/zconf.h uses this */

/*
// bool defs pulled from the Perl5 Source, Copyright 1991-1994, Larry Wall
*/

#ifdef _G_HAVE_BOOL
#  if _G_HAVE_BOOL
#    ifndef HAS_BOOL
#      define HAS_BOOL 1
#    endif
#  endif
#endif

#ifndef HAS_BOOL
#  ifdef UTS      /* what is UTS, need documentation -- BJG */
#    define bool int
#  else
#    define bool char 
#  endif
#endif

typedef bool              Bool;

#if SIZEOF_CHAR == 1
  typedef char               Byte;
  typedef unsigned char      uByte;
  typedef char               Char;
  typedef unsigned char      uChar;
#elif SIZEOF_SHORT == 1
  typedef short int          Byte;
  typedef unsigned short int uByte;
  typedef char               Char;
  typedef unsigned char      uChar;
#else
# error "Unable to specify size for Byte and Char type (8 bits)"
#endif

#if SIZEOF_SHORT == 2
  typedef short int          Short;
  typedef unsigned short int uShort;
#elif SIZEOF_CHAR == 2   /* unlikely */
  typedef char               Short;
  typedef unsigned char      uShort;
#elif SIZEOF_INT == 2
  typedef int                Short;
  typedef unsigned int       uShort;
#else
# error "Unable to specify size for Short type (16 bits)"
#endif
# define MAX_SHORT  32767
# define MAX_USHORT 65535

 /* when monkeys fly */
#if SIZEOF_SHORT == 4
  typedef short int          Int;
  typedef unsigned short int uInt;
#elif SIZEOF_INT == 4
  typedef int               Int;
  typedef unsigned int      uInt;
# define MAX_INT  2147483647
# define MAX_UINT 4294967295U
#elif SIZEOF_LONG == 4
  typedef long              Int;
  typedef unsigned long     uInt; 
# define MAX_INT  2147483647L
# define MAX_UINT 4294967295UL
#else
# error "Unable to specify size for Int type (32 bits)"
#endif

/*
#if SIZEOF_LONG == 8
*/
   typedef long              Long;
   typedef unsigned long     uLong;
#  define MAX_LONG  9223372036854775807L
#  define MAX_ULONG 18446744073709551615UL
/*
#elif SIZEOF_LLONG == 8
   typedef long long         Long;
   typedef unsigned long long uLong;
#  define MAX_LONG  9223372036854775807LL
#  define MAX_ULONG 18446744073709551615ULL
#else
#  error "Unable to specify size for Long type (64 bits)"
#endif
*/

#if SIZEOF_FLOAT == 4
   typedef float           Float; /* should always be this */
#elif SIZEOF_DOUBLE == 4   /* will this ever occur !? */
   typedef double          Float;
#else
#  error "Unable to specify size for Float type (32 bits)"
#endif

#if SIZEOF_FLOAT == 8     /* not likely */
   typedef float           Double;
#elif SIZEOF_DOUBLE == 8
   typedef double          Double;
#elif SIZEOF_LDOUBLE == 8
   typedef long double     Double; /* not ANSI */
#else
#  error "Unable to specify size for Double type (64 bits)"
#endif

